/* BdfServer_Get - Copyright (c) 2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.get.access.v1;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.get.access.AccessGate;
import fr.exemole.bdfserver.get.access.AccessJsonProducer;
import fr.exemole.bdfserver.tools.instruction.BdfErrors;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.roles.PermissionSummaryBuilder;
import fr.exemole.bdfserver.tools.subsettree.TreeUtils;
import java.util.Collection;
import java.util.Comparator;
import java.util.function.Predicate;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.Fiches;
import net.fichotheque.exportation.access.AccessDef;
import net.fichotheque.permission.PermissionSummary;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.tools.corpus.FichesBuilder;
import net.fichotheque.tools.selection.SelectionEngines;
import net.fichotheque.utils.ThesaurusUtils;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class V1EndPointFactory {

    private V1EndPointFactory() {

    }

    public static JsonProducer getJsonProducer(BdfServer bdfServer, RequestMap requestMap, AccessDef accessDef, Lang lang, String endPoint, @Nullable BdfUser bdfUser) throws ErrorMessageException {
        PermissionSummary permissionSummary = null;
        if (bdfUser != null) {
            permissionSummary = PermissionSummaryBuilder.build(bdfServer, bdfUser);
        }
        AccessGate accessGate = AccessGate.build(bdfServer, accessDef, permissionSummary, lang);
        AccessJsonProducer accessJsonProducer = new AccessJsonProducer();
        switch (endPoint) {
            case "fiche": {
                FicheMeta ficheMeta = getFiche(accessGate, requestMap);
                accessJsonProducer.add(new FicheJsonProperty(accessGate.getPermissionSummary(), ficheMeta, accessGate.getCellEngine()));
                break;
            }
            case "fiches": {
                Fiches fiches = getFiches(accessGate, requestMap);
                accessJsonProducer.add(new FichesJsonProperty(accessGate.getPermissionSummary(), fiches, accessGate.getCellEngine(), lang));
                break;
            }
            case "thesaurus": {
                Thesaurus thesaurus = getThesaurus(accessGate, requestMap);
                accessJsonProducer.add(new ThesaurusJsonProperty(thesaurus, accessGate.getCellEngine(), lang, ThesaurusUtils.checkDisponibility(accessGate.getBdfServer().getThesaurusLangChecker(), thesaurus, lang)));
                break;
            }
            case "fiche-array": {
                Collection<FicheMeta> collection = getFicheArray(accessGate, requestMap);
                accessJsonProducer.add(new CountJsonProperty(collection));
                accessJsonProducer.add(new FicheArrayJsonProperty(collection, accessGate.getPermissionSummary(), accessGate.getCellEngine()));
                break;
            }
            case "motcle-array": {
                Collection<Motcle> collection = getMotcleArray(accessGate, requestMap);
                accessJsonProducer.add(new CountJsonProperty(collection));
                accessJsonProducer.add(new MotcleArrayJsonProperty(collection, accessGate.getCellEngine(), lang));
                break;
            }
        }
        if (accessJsonProducer.isEmpty()) {
            return null;
        } else {
            return accessJsonProducer;
        }
    }

    private static FicheMeta getFiche(AccessGate accessGate, RequestMap requestMap) throws ErrorMessageException {
        FicheMeta ficheMeta = BdfInstructionUtils.getMandatoryFicheMeta(accessGate.getBdfServer().getFichotheque(), requestMap);
        if (!accessGate.getFichePredicate().test(ficheMeta)) {
            throw BdfErrors.unknownParameterValue(InteractionConstants.ID_PARAMNAME, String.valueOf(ficheMeta.getId()));
        }
        return ficheMeta;
    }

    private static Fiches getFiches(AccessGate accessGate, RequestMap requestMap) throws ErrorMessageException {
        SelectionContext selectionContext = accessGate.getSelectionContext();
        BdfServer bdfServer = accessGate.getBdfServer();
        String selection = requestMap.getParameter(InteractionConstants.SELECTION_PARAMNAME);
        String xml = requestMap.getParameter(InteractionConstants.XML_PARAMNAME);
        FichesBuilder fichesBuilder = FichesBuilder.init(BdfInstructionUtils.getComparator(accessGate.getLang(), requestMap))
                .initSubsetKeyOrder(TreeUtils.getCorpusKeyList(bdfServer));
        if ((selection != null) || (xml != null)) {
            FicheSelectorBuilder selectorBuilder = FicheSelectorBuilder.init(selectionContext);
            if (selection != null) {
                BdfInstructionUtils.populateFromSelection(bdfServer, requestMap, selectorBuilder);
            } else {
                BdfInstructionUtils.populateFromXml(bdfServer, requestMap, selectorBuilder);
            }
            fichesBuilder.populate(selectorBuilder.toFicheSelector());
        } else {
            Predicate<FicheMeta> fichePredicate = accessGate.getFichePredicate();
            for (Corpus corpus : bdfServer.getFichotheque().getCorpusList()) {
                if (accessGate.getSubsetPredicate().test(corpus)) {
                    for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                        if (fichePredicate.test(ficheMeta)) {
                            fichesBuilder.add(ficheMeta);
                        }
                    }
                }
            }
        }
        return fichesBuilder.toFiches();
    }

    private static Thesaurus getThesaurus(AccessGate accessGate, RequestMap requestMap) throws ErrorMessageException {
        Thesaurus thesaurus = BdfInstructionUtils.getMandatoryThesaurus(accessGate.getBdfServer().getFichotheque(), requestMap);
        if (!accessGate.getSubsetPredicate().test(thesaurus)) {
            throw BdfErrors.unknownParameterValue(SubsetKey.CATEGORY_THESAURUS_STRING, thesaurus.getSubsetName());
        }
        return thesaurus;
    }

    private static Collection<FicheMeta> getFicheArray(AccessGate accessGate, RequestMap requestMap) throws ErrorMessageException {
        SelectionContext selectionContext = accessGate.getSelectionContext();
        Comparator<FicheMeta> comparator = BdfInstructionUtils.getComparator(accessGate.getLang(), requestMap);
        FicheSelectorBuilder selectorBuilder = FicheSelectorBuilder.init(selectionContext);
        BdfInstructionUtils.populateFromXml(accessGate.getBdfServer(), requestMap, selectorBuilder);
        return SelectionEngines.run(selectionContext, selectorBuilder.toFicheSelector(), comparator);
    }

    private static Collection<Motcle> getMotcleArray(AccessGate accessGate, RequestMap requestMap) throws ErrorMessageException {
        SelectionContext selectionContext = accessGate.getSelectionContext();
        MotcleSelectorBuilder selectorBuilder = MotcleSelectorBuilder.init(selectionContext);
        BdfInstructionUtils.populateFromXml(accessGate.getBdfServer(), requestMap, selectorBuilder);
        return SelectionEngines.run(selectionContext, null, selectorBuilder.toMotcleSelector(), null);
    }

}
